/**********************************************************************
* @file		A31G216X_adc.c
* @brief	Contains all functions support for ADC firmware library on A31G216X
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2015, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A31G31x_comp.h"
#include "A31G31x_scu.h"
#include "debug_frmwrk.h"


/***********************************************************************************************
 * @brief		Comparator Initialize
 ***********************************************************************************************/
void CMP_Init(void){
	SCU->PER2 &= ~(1<<23);
	SCU->PCER2 &= ~(1<<23);
	
	SCU->PER2 |= (1<<23);
	SCU->PCER2 |= (1<<23);		
}


/***********************************************************************************************
 * @brief		Get Comparator Interrupt Status
 * @param[in]	Comparator selection, should be in range from CMP0, CMP1
 * @return	IST register value
 ***********************************************************************************************/
uint32_t CMP_GetIntStatusSelect(uint32_t comp_no)
{
	if(comp_no==CMP0){
		return CMP->IST & 0x01;
	}
	else {
		return CMP->IST & 0x02;
	}
}

/***********************************************************************************************
 * @brief		Get All Comparator interrupt status
 * @param[in]	None
 * @return	IST register value
 ***********************************************************************************************/
uint32_t CMP_GetIntStatusAll()
{
	return CMP->IST;
}

/***********************************************************************************************
 * @brief		Clear Comparator interrupt status 
 * @param[in]	Comparator selection, should be in range from CMP0, CMP1
 * @return	None
 ***********************************************************************************************/
void CMP_ClearIntSelect(uint32_t comp_no)
{
	if(comp_no==CMP0){
		CMP->ICLR = CMP_ICLR_C0ICLR ;
	}
	else{
		CMP->ICLR = CMP_ICLR_C1ICLR;
	}
}

/***********************************************************************************************
 * @brief		Clear All Comparator interrupt status
 * @param[in]	None
 * @return	None
 ***********************************************************************************************/
void CMP_ClearIntAll()
{
	CMP->ICLR = CMP_ICLR_BITMASK;
}

/***********************************************************************************************
 * @brief		Configure comparator hysteris 
* @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
* @param[in]	hyssel		comparator hysterisis select, should be :
						- CMP_HYSSEL_5MV	= 0,   					
						- CMP_HYSSEL_20MV	= 1
* @param[in]	NewState	 	should be :
* 						- DISABLE= 0
* 						- ENABLE	= 1
 * @return	None
 ***********************************************************************************************/
void CMP_HysterisisConfigure(uint32_t comp_no, uint32_t hyssel, FunctionalState NewState)
{
	if(comp_no==CMP0){
		CMP->CMP0CR = (CMP->CMP0CR & ~(CMP_CR_HYSEN | CMP_CR_HYSSEL_BITMASK));
		if(NewState == ENABLE){
			CMP->CMP0CR |= (hyssel<<16) | (CMP_CR_HYSEN<<20);
		}
	}
	else{
		CMP->CMP1CR = (CMP->CMP1CR & ~(CMP_CR_HYSEN | CMP_CR_HYSSEL_BITMASK));
		if(NewState == ENABLE){
			CMP->CMP1CR |= (hyssel<<16) | (CMP_CR_HYSEN<<20);
		}
	}
}
	
/***********************************************************************************************
* @brief		Configure comparator reference/input voltage
* @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
* @param[in]	refsel		Comparator Reference(input -) Selection bit, should be :
						- CMP_INPUT_CP1A	= 0,   					
						- CMP_INPUT_CP0,CMP_INPUT_CP1A =1
						- CMP_INPUT_CP1B	= 2
						- CMP_INPUT_CP1C	= 3
* @param[in]	inputsel		Comparator Input(input +) Selection bit, should be :
						- CMP_REF_CREF		= 0  					
						- CMP_REF_BGR1V	= 1		 							
						- CMP_REF_DACOUT	= 2
* @return	None
 ***********************************************************************************************/
void CMP_ConfigureFunction(uint32_t comp_no, uint32_t refsel, uint32_t inputsel)
{
	if(comp_no==CMP0){
		CMP->CMP0CR = (CMP->CMP0CR & ~(CMP_CR_INPSEL_BITMASK | CMP_CR_INNSEL_BITMASK));
		CMP->CMP0CR |= (inputsel) | (refsel<<4);
		}
	else{
		CMP->CMP1CR = (CMP->CMP1CR & ~((CMP_CR_INPSEL_BITMASK | CMP_CR_INNSEL_BITMASK)));
		CMP->CMP1CR |= (inputsel) | (refsel<<4);
	}
}

/***********************************************************************************************
* @brief		Enable/disable Comparator operate
* @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
* @param[in]	NewState	 	should be :
* 						- DISABLE= 0
* 						- ENABLE	= 1
 * @return	None
 ***********************************************************************************************/
void CMP_Cmd(uint32_t comp_no,  FunctionalState NewState)
{
	if(comp_no==CMP0){
		CMP->CMP0CR = (CMP->CMP0CR & ~CMP_CR_EN);
		if(NewState == ENABLE){
			CMP->CMP0CR |= CMP_CR_EN;
		}
	}
	else{
		CMP->CMP1CR = (CMP->CMP1CR & ~CMP_CR_EN);
		if(NewState == ENABLE){
			CMP->CMP1CR |= CMP_CR_EN;
		}
	}
}

/***********************************************************************************************
 * @brief		Set Comparator Debounce 
 * @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
 * @param[in]	dbncshift		Debounce shift selection, should be in range from 0 to 0xF
 * @param[in]	dbnccounter	Debounce time base counter, should be in range from 0 to 0xFFFF
 * @return		None
 ***********************************************************************************************/
void CMP_Debounce(uint32_t comp_no, uint32_t dbncshift, uint32_t dbnccounter)
{
	if(comp_no==CMP0){
		CMP->DBNC &=  ~(CMP_DBNC_C0DBNC_BITMASK | CMP_DBNC_DBNCTB_BITMASK);
		CMP->DBNC = CMP->DBNC | (dbncshift << 0);
		CMP->DBNC = CMP->DBNC | (dbnccounter << 16);
	}
	else{
		CMP->DBNC &= ~(CMP_DBNC_C1DBNC_BITMASK | CMP_DBNC_DBNCTB_BITMASK);
		CMP->DBNC = CMP->DBNC | (dbncshift << 4);
		CMP->DBNC = CMP->DBNC | (dbnccounter << 16);
	}
}

/***********************************************************************************************
* @brief		Enable/disable Comparator Interrupt
* @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
* @param[in]	NewState	 	should be :
* 						- DISABLE = 0
* 						- ENABLE	= 1
* @return	None
 ***********************************************************************************************/
void CMP_IntCmd(uint32_t comp_no, FunctionalState NewState)
{
	if(comp_no==CMP0){
		if(NewState == ENABLE){
			CMP->IEN |= CMP_IEN_C0_EN;
		}
		else{
			CMP->IEN &= ~CMP_IEN_C0_EN;
		}
	}
	else{
		if(NewState == ENABLE){
			CMP->IEN |= CMP_IEN_C1_EN;
		}
		else{
			CMP->IEN &= ~CMP_IEN_C1_EN;
		}
	}
}

/***********************************************************************************************
* @brief		Configure comparator interrupt 
* @param[in]	comp_no		Comparator selection, should be in range from CMP0, CMP1
* @param[in]	tpol			Comparator Trigger output polarity(to trigger other IP), should be :
						- CMP_TRIOUT_NORMAL		= 0   					
						- CMP_TRIOUT_INVERTED		= 1
* @param[in]	ipol			Comparator interrupt polarity(level mode), should be :
						- CMP_INTOUT_HIGH		= 0 					
						- CMP_INTOUT_LOW		= 1
* @param[in]	mode		Comparator Interrupt mode select, should be :
						- ICON_LEVEL				= 0     		
						- ICON_RISING_EDGE_INT	= 1		     			
						- ICON_FALLING_EDGE_INT	= 2		     		
						- ICON_BOTH_EDGE_INT		= 3
 * @return	None
 ***********************************************************************************************/
void CMP_IntConfigure(uint32_t comp_no, uint32_t tpol, uint32_t ipol, uint32_t mode)
{
	if(comp_no==CMP0){
		CMP->ICON = CMP->ICON & ~(CMP_ICON_IPOL0_BITMASK | CMP_ICON_TPOL0_BITMASK | CMP_ICON_C0IMODE_BITMASK);
		CMP->ICON |= (tpol << 12);
		CMP->ICON |= (ipol << 8);
		CMP->ICON |= (mode << 0);
	}
	else{
		CMP->ICON = CMP->ICON & ~(CMP_ICON_IPOL1_BITMASK | CMP_ICON_TPOL1_BITMASK | CMP_ICON_C1IMODE_BITMASK);
		CMP->ICON |= (tpol << 13);
		CMP->ICON |= (ipol << 9);
		CMP->ICON |= (mode << 2);
	}
}
/* --------------------------------- End Of File ------------------------------ */

