#include "A31G31x_pcu.h"
#include "A31G31x_scu.h"
#include "A31G31x_uartn.h"
#include "debug_frmwrk.h"

//-------------------------
#define USED_CLKO
//-------------------------
#define USED_HSI
//#define USED_LSI
//#define USED_MOSC
//#define USED_SOSC
//#define USED_MOSCPLL
//#define USED_HSIPLL

/* buffer size definition */
#define UART_RING_BUFSIZE 32

/* Buf mask */
#define __BUF_MASK (UART_RING_BUFSIZE-1)
/* Check buf is full or not */
#define __BUF_IS_FULL(head, tail) ((tail&__BUF_MASK)==((head+1)&__BUF_MASK))
/* Check buf will be full in next receiving or not */
#define __BUF_WILL_FULL(head, tail) ((tail&__BUF_MASK)==((head+2)&__BUF_MASK))
/* Check buf is empty */
#define __BUF_IS_EMPTY(head, tail) ((head&__BUF_MASK)==(tail&__BUF_MASK))
/* Reset buf */
#define __BUF_INCR(bufidx)	(bufidx=(bufidx+1)&__BUF_MASK)

typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART_RING_BUFFER_T;

// UART Ring buffer
UART_RING_BUFFER_T rb;

// Current Tx Interrupt enable state
__IO FlagStatus TxIntStat;

/* Interrupt service routines */
void UART0_IRQHandler(void);
void UART1_IRQHandler(void);
void UART_IntErr(uint8_t bLSErrType);
void UART_IntTransmit(UART_Type *UARTx);
void UART_IntReceive(UART_Type *UARTx);

uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen);
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen);

// system default I/O setting
void PCU_Init(void);
void SCU_ClockInit(void);


/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		UART1 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void UART1_IRQHandler(void)
{
	uint32_t intsrc, tmp, tmp1;

	/* Determine the interrupt source */
	intsrc = UART1->IIR;
	tmp = intsrc & UART_IIR_INTID_MASK;

	// Receive Line Status
	if (tmp == UART_IIR_INTID_RLS){
		// Check line status
		tmp1 = UART1->LSR;
		// Mask out the Receive Ready and Transmit Holding empty status
		tmp1 &= (UART_LSR_OE | UART_LSR_PE | UART_LSR_FE | UART_LSR_BI);
		// If any error exist
		if (tmp1) {
				UART_IntErr(tmp1);
		}
	}

	// Receive Data Available or Character time-out
	if (tmp == UART_IIR_INTID_RDA){
			UART_IntReceive(UART1);
	}

	// Transmit Holding Empty
	if (tmp == UART_IIR_INTID_THRE){
			UART_IntTransmit(UART1);
	}
}


/********************************************************************//**
 * @brief 		UART receive function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntReceive(UART_Type *UARTx)
{
	uint8_t tmpc;
	uint32_t rLen;

	while(1){
		// Call UART read function in UART driver
		rLen = UART_Receive((UART_Type *)UARTx, &tmpc, 1, NONE_BLOCKING);
		// If data received
		if (rLen){
			/* Check if buffer is more space
			 * If no more space, remaining character will be trimmed out
			 */
			if (!__BUF_IS_FULL(rb.rx_head,rb.rx_tail)){
				rb.rx[rb.rx_head] = tmpc;
				__BUF_INCR(rb.rx_head);
			}
		}
		// no more data
		else {
			break;
		}
	}
}

/********************************************************************//**
 * @brief 		UART transmit function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntTransmit(UART_Type *UARTx)
{
    // Disable THRE interrupt
    UART_IntConfig((UART_Type *)UARTx, UART_INTCFG_THRE, DISABLE);

	/* Wait until THR empty */
    while (UART_CheckBusy((UART_Type *)UARTx) == SET);

	while (!__BUF_IS_EMPTY(rb.tx_head,rb.tx_tail))
    {
        /* Move a piece of data into the transmit FIFO */
    	if (UART_Send((UART_Type *)UARTx, (uint8_t *)&rb.tx[rb.tx_tail], 1, NONE_BLOCKING)){
        /* Update transmit ring FIFO tail pointer */
        __BUF_INCR(rb.tx_tail);
    	} else {
    		break;
    	}
    }

    /* If there is no more data to send, disable the transmit
       interrupt - else enable it or keep it enabled */
	if (__BUF_IS_EMPTY(rb.tx_head, rb.tx_tail)) {
    	UART_IntConfig((UART_Type *)UARTx, UART_INTCFG_THRE, DISABLE);
    	// Reset Tx Interrupt state
    	TxIntStat = RESET;
    }
    else
		{
      	// Set Tx Interrupt state
		TxIntStat = SET;
    	UART_IntConfig((UART_Type *)UARTx, UART_INTCFG_THRE, ENABLE);
    }
}


/*********************************************************************//**
 * @brief		UART Line Status Error
 * @param[in]	bLSErrType	UART Line Status Error Type
 * @return		None
 **********************************************************************/
void UART_IntErr(uint8_t bLSErrType)
{
	uint8_t test;
	// Loop forever
	while (1){
		// For testing purpose
		test = bLSErrType;
		test = test;
	}
}

/*********************************************************************//**
 * @brief		UART transmit function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART1
 * @param[out]	txbuf Pointer to Transmit buffer
 * @param[in]	buflen Length of Transmit buffer
 * @return 		Number of bytes actually sent to the ring buffer
 **********************************************************************/
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) txbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART transmit interrupts during this
	   read so the UART transmit interrupt won't cause problems
	   with the index values */
    UART_IntConfig(UARTx, UART_INTCFG_THRE, DISABLE);

	/* Loop until transmit run buffer is full or until n_bytes
	   expires */
	while ((buflen > 0) && (!__BUF_IS_FULL(rb.tx_head, rb.tx_tail)))
	{
		/* Write data from buffer into ring buffer */
		rb.tx[rb.tx_head] = *data;
		data++;

		/* Increment head pointer */
		__BUF_INCR(rb.tx_head);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/*
	 * Check if current Tx interrupt enable is reset,
	 * that means the Tx interrupt must be re-enabled
	 * due to call UART_IntTransmit() function to trigger
	 * this interrupt type
	 */
	if (TxIntStat == RESET) {
		UART_IntTransmit(UARTx);
	}
	/*
	 * Otherwise, re-enables Tx Interrupt
	 */
	else {
		UART_IntConfig(UARTx, UART_INTCFG_THRE, ENABLE);
	}

    return bytes;
}


/*********************************************************************//**
 * @brief		UART read function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART1
 * @param[out]	rxbuf Pointer to Received buffer
 * @param[in]	buflen Length of Received buffer
 * @return 		Number of bytes actually read from the ring buffer
 **********************************************************************/
uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) rxbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART receive interrupts during this
	   read so the UART receive interrupt won't cause problems
	   with the index values */
	UART_IntConfig(UARTx, UART_INTCFG_RBR, DISABLE);

	/* Loop until receive buffer ring is empty or
		until max_bytes expires */
	while ((buflen > 0) && (!(__BUF_IS_EMPTY(rb.rx_head, rb.rx_tail))))
	{
		/* Read data from ring buffer into user buffer */
		*data = rb.rx[rb.rx_tail];
		data++;

		/* Update tail pointer */
		__BUF_INCR(rb.rx_tail);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/* Re-enable UART interrupts */
	UART_IntConfig(UARTx, UART_INTCFG_RBR, ENABLE);

    return bytes;
}


void mainloop(void)
{
	UART_CFG_Type UARTConfigStruct;	
	uint32_t idx, len;
	uint8_t buffer[10];

	/*
	 * Initialize UART1 pin connect
	 */
	PCU_SetDirection(PB, 7, ALTERN_FUNC);
	PCU_ConfigureFunction(PB, 7, FUNC1);
	PCU_ConfigurePullupdown(PB, 7, ENPU);
	
	PCU_SetDirection(PB, 6, ALTERN_FUNC);
	PCU_ConfigureFunction(PB, 6, FUNC1);

	// default: 38400-8-N-1	
	UART_ConfigStructInit(&UARTConfigStruct);	
	UART_Init(UART1, &UARTConfigStruct);

	    /* Enable UART Rx interrupt */
	UART_IntConfig((UART_Type *)UART1, UART_INTCFG_RBR, ENABLE);
	/* Enable UART line status interrupt */
	UART_IntConfig((UART_Type *)UART1, UART_INTCFG_RLS, ENABLE);
	
	TxIntStat = RESET;
	
	// Reset ring buf head and tail idx
	rb.rx_head=0;
	rb.rx_tail=0;
	rb.tx_head=0;
	rb.tx_tail=0;

	NVIC_SetPriority(UART1_IRQn, 3);
	NVIC_EnableIRQ(UART1_IRQn);
		
	__enable_irq();
	
	UARTSend(UART1,(uint8_t *)"Interrupt test \n\r",17);	
	while(1){
       len = 0;
        while (len == 0)
        {
            len = UARTReceive((UART_Type *)UART1, buffer, sizeof(buffer));
        }

        /* Got some data */
        idx = 0;
        while (idx < len)
        {
            if (buffer[idx] == 0x0d)
            {
//                UARTSend((UART_Type *)UART1, (uint8_t *)"\n\r", 2);
                UARTSend((UART_Type *)UART1, (uint8_t *)"\n\r 123456789012334567890", (2+22));				
            }
            else
            {
                /* Echo it back */
            	UARTSend((UART_Type *)UART1, &buffer[idx], 1);
				UARTSend((UART_Type *)UART1, (uint8_t *)"[1234567890123345678] \n\r", (2+22));	
            }
            idx++;
        }
	}
}

int main(void)
{
	__disable_irq();
	SystemInit();
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
	SCU_ClockInit(); 

	debug_frmwrk_init();
	__enable_irq();
	
	mainloop();
	
	return (0);
}

/**********************************************************************
 * @brief		Initialize default clock for A31G31x
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SCU_ClockInit(void)
{
	uint32_t i;

//	CLKO function setting. check PORT setting (PF4).

#ifdef USED_CLKO
	SCU_SetCOR(4,ENABLE); //    /10
#else
	SCU_SetCOR(4,DISABLE);
#endif

	SCU->CMR&=~(1<<7); //mclk monitoring disable
	
#ifdef USED_LSI			//500khz
	SCU_SetLSIOSC(LSIOSC_EN); //LSIOSC_EN_DIV2, LSIOSC_EN_DIV4
	SystemCoreClock=500000; //500khz
	SystemPeriClock=500000; //500khz	
	
	for (i=0;i<10;i++);	
	
	SCU_ChangeSysClk(SC_LSIOSC);
#endif 
	
#ifdef USED_SOSC  //32.768khz
	SCU_SetSOSC(EXSOSC_EN);
	SystemCoreClock=32768; //32.768khz
	SystemPeriClock=32768; //32.768khz	

	for (i=0;i<100;i++);	

// wait for SOSC stable	
	SCU_WaitForSOSCStartUp();
	SCU_ChangeSysClk(SC_SOSC);
#endif 	

#ifdef USED_HSI		//48Mhz
	SCU_SetHSIOSC(HSIOSC_EN);
	SystemCoreClock=48000000; //48Mhz
	SystemPeriClock=48000000; //48Mhz	
	
	for (i=0;i<10;i++);	

	SCU_ChangeSysClk(SC_HSIOSC);
#endif 	

#ifdef USED_MOSC	//xMhz
	SCU_SetMOSC(EXOSC_EN);
	SystemCoreClock=8000000; //xMhz
	SystemPeriClock=8000000; //xMhz	

	for (i=0;i<100;i++);	

	SCU_WaitForMOSCStartUp();
	SCU_ChangeSysClk(SC_EXOSC);
#endif


#ifdef USED_MOSCPLL
// PLL setting 
//    FIN=PLLINCLK/(R+1)                                             ; R: Pre Divider   
//    FOUT=(FIN*(N1+1)*(D+1))  / ((N2+1)*(P+1))          ; N1: Post Divider1, N2:Post Divider2, P:Output Divider,      
//             = FVCO *(D+1)                                             ; D:Frequency Doubler
//
//ex)    FIN=PLLINCLK/(R+1) = 8M/(3+1) = 2M                               ; R:3, PLLINCLK:8MHz(MOSC)
//         FOUT=(2M*(47+1)*(0+1)) / ((1+1)*(0+1) = 48MHz              ; N1:47, D:0, N2:1, P:0
//
	if (SCU_SetPLLandWaitForPLLStartUp(ENABLE,  
		PLLCON_BYPASS_PLL,    //PLLCON_BYPASS_FIN:0, PLLCON_BYPASS_PLL:1
		0,                                    //0:FOUT==VCO, 1:FOUT==2xVCO,  D=0
//		3,                                    //PREDIV, R=3
		1,                                    //PREDIV, R=1
		47,                                  //POSTDIV1, N1=47  
		1,                                    //POSTDIV2, N2=1
		0)==ERROR)                    //OUTDIV P=0
	{
		while(1);
	}

//	EOSC -->  EOSCPLL
	SCU_ChangeSysClk(SC_EXOSCPLL);
	
	SystemCoreClock=48000000; 
	SystemPeriClock=48000000; 
#endif

	
#ifdef USED_HSIPLL
// PLL setting 
//    FIN=PLLINCLK/(R+1)                                             ; R: Pre Divider   
//    FOUT=(FIN*(N1+1)*(D+1))  / ((N2+1)*(P+1))          ; N1: Post Divider1, N2:Post Divider2, P:Output Divider,      
//             = FVCO *(D+1)                                             ; D:Frequency Doubler
//
//ex)    FIN=PLLINCLK/(R+1) = 8M/(3+1) = 2M                               ; R:3, PLLINCLK:8MHz(MOSC)
//         FOUT=(2M*(47+1)*(0+1)) / ((1+1)*(0+1) = 48MHz              ; N1:47, D:0, N2:1, P:0
//
	if (SCU_SetPLLandWaitForPLLStartUp(ENABLE,  
		PLLCON_BYPASS_PLL,    //PLLCON_BYPASS_FIN:0, PLLCON_BYPASS_PLL:1
		0,                                    //0:FOUT==VCO, 1:FOUT==2xVCO,  D=0
//		3,                                    //PREDIV, R=3
		1,                                    //PREDIV, R=1
		47,                                  //POSTDIV1, N1=47  
		1,                                    //POSTDIV2, N2=1
		0)==ERROR)                    //OUTDIV P=0
	{
		while(1);
	}

//	EOSC -->  EOSCPLL
	SCU_ChangeSysClk(SC_HSIOSCPLL);
	
	SystemCoreClock=48000000; 
	SystemPeriClock=48000000; 
#endif

	SCU->CMR|=(1<<7); //mclk monitoring enable

// wait setting order 1. default wait setting -> 2. clock change -> 3. adjust wait setting
//// flash memory controller
	FMC->MR = 0x81;       // after changing 0x81 -> 0x28 in MR reg, flash access timing will be able to be set.
	FMC->MR = 0x28;       // enter flash access timing changing mode
//	FMC->CFG = (0x7858<<16) | (0<<8);  //flash access cycles 	20
//	FMC->CFG = (0x7858<<16) | (1<<8);  //flash access cycles 	40
	FMC->CFG = (0x7858<<16) | (2<<8);  //flash access cycles 	60		
//	FMC->CFG = (0x7858<<16) | (3<<8);  //flash access cycles 		
	                              // flash access time cannot overflow 20Mhz.
	                              // ex) if MCLK=48Mhz, 
	                              //       48/1 = 48 (can't set no wait)
	                              //       48/2 = 24 (1 wait is not ok)
	                              //       48/3 = 16 (2 wait is ok)								  
	                              // so, 2 wait is possible.
	FMC->MR = 0;	      // exit flash access timing --> normal mode				
}

void PCU_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<13)   // GPIOF
	| (1<<12)   // GPIOE
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<13)   // GPIOF
	| (1<<12)   // GPIOE
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	

 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	//--------------------------------------------------------------
	//	PORT INIT
	//		PA, PB, PC, PD, PE, PF
	//--------------------------------------------------------------
	// PORT - A
	PA->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9

	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x01<<8)              // P4
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PA->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9

	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PA->AFSR1 = 0
	| (0x00<<28)              // P7	  0 : SEG 42,	1 : 			, 2 :				, 3 : AN7/CREF0/DAVREF	, 4 : ISEG2	
	| (0x00<<24)              // P6		0 : SEG 43,	1 : 			, 2 :				, 3 : AN6/CREF1/DAO			, 4 : ISEG3	
	| (0x00<<20)              // P5		0 :				,	1 : T12O	, 2 : T12C	, 3 : AN5/CP1A					, 4 : ISEG4
	| (0x00<<16)              // P4		0 :				,	1 : 			, 2 :				, 3 : AN4/CP1B					, 4 :CS7/ISEG5
	| (0x00<<12)              // P3		0 : 			,	1 : 			, 2 :				, 3 : AN3/CP1C					, 4 :CS6/ISEG6
	| (0x00<<8)              // P2		0 : 			,	1 : EC12	, 2 :				, 3 : AN2/AVREF/CP0			, 4 :CS5/ISEG7
	| (0x00<<4)              // P1		0 : 			,	1 : SCL1	, 2 :				, 3 : AN1/USBDP					, 4 :CS4/ISEG8
	| (0x00<<0)              // P0		0 : 			,	1 : SDA1	, 2 :				, 3 : AN0/USBDM					, 4 :CS3/ISEG9
	;

	PA->AFSR2 = 0
	| (0x00<<12)              // P11	0 : 			,	1 : 			, 2 :				, 3 : AN14					, 4 : CS2	
	| (0x00<<8)              // P10		0 : 			,	1 : 			, 2 :				, 3 : AN13					, 4 : CS1	 
	| (0x00<<4)              // P9		0 : 			,	1 : 			, 2 :				, 3 : AN12					, 4 : CS0	 
	;
	
	PA->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9

	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PA->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9

	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - B
	PB->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<30)              // P15
	| (0x01<<28)              // P14
	| (0x01<<26)              // P13
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x02<<10)              // P5   SWDIO
	| (0x02<<8)              // P4   SWCLK
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PB->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<15)              // P15
	| (0x00<<14)              // P14
	| (0x00<<13)              // P13
	| (0x00<<12)              // P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PB->AFSR1 = 0
	| (0x01<<28)              // P7		0 : SEG 34,	1 : RXD1	, 2 : 				, 3 :						, 4 : ICOM19 
	| (0x01<<24)              // P6		0 : SEG 35,	1 : TXD1	, 2 :					, 3 :						, 4 : ICOM20
	| (0x02<<20)              // P5		0 : SEG 36,	1 : RXD0	, 2 : SWDIO		, 3 :						, 4 : ICOM21 
	| (0x02<<16)              // P4		0 : SEG 37,	1 : TXD0	, 2 : SWCLK		, 3 :						, 4 : ICOM22 
	| (0x01<<12)              // P3		0 : SEG 38,	1 : BOOT	, 2 : SS10		, 3 :						, 4 : ICOM23
	| (0x00<<8)              // P2			0 : SEG 39,	1 : 			, 2 : SCK10		, 3 : AN10			, 4 : CS10/ICOM24 
	| (0x00<<4)              // P1			0 : SEG 40,	1 : RXD10	, 2 : MISO10	, 3 : AN9				, 4 : CS9/ICOM25/ISEG0
	| (0x00<<0)              // P0			0 : SEG 41,	1 : TXD10	, 2 : MOSI10	, 3 : AN8				, 4 : CS8/ICOM26/ISEG1
	;

	PB->AFSR2 = 0
	| (0x0<<28)              // P15		0 : SEG 26,	1 :				, 2 :					, 3 : 				, 4 :	CS18		 
	| (0x0<<24)              // P14		0 : SEG 27,	1 : 			, 2 :					, 3 : 				, 4 :	CS17		 
	| (0x0<<20)              // P13		0 : SEG 28,	1 :				, 2 :					, 3 : 				, 4 :	CS16		 
	| (0x0<<16)              // P12		0 : SEG 29,	1 :				, 2 :					, 3 : 				, 4 :	CS15		 
	| (0x0<<12)              // P11		0 : SEG 30,	1 : T15C	, 2 : EC16		, 3 : 				, 4 : CS14/T15O	 
	| (0x0<<8)              // P10		0 : SEG 31,	1 : T16C	, 2 : EC15		, 3 :	    		, 4 : CS13/T16O	 
	| (0x0<<4)              // P9			0 : SEG 32,	1 : T16O	, 2 : T16C		, 3 :     		, 4 : CS12/EC15	 
	| (0x0<<0)              // P8			0 : SEG 33,	1 : T15O	, 2 : T15C		, 3 :     		, 4 : CS11/EC16	 
	;
	
	PB->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<30)              // P15
	| (0x0<<28)              // P14
	| (0x0<<26)              // P13
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PB->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x00<<15)              // P15
	| (0x00<<14)              // P14
	| (0x00<<13)              // P13
	| (0x00<<12)              // P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8	
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - C
	PC->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x01<<8)              // P4
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PC->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<12)              // P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PC->AFSR1 = 0
	| (0x0<<28)              // P7		0 : SEG 18	, 1 :					, 2 :					, 3 :       , 4 :				
	| (0x0<<24)              // P6		0 : SEG 19	, 1 : SCL2		, 2 :					, 3 :       , 4 :				
	| (0x0<<20)              // P5		0 : SEG 20	, 1 : SDA2		, 2 :					, 3 :       , 4 :				
	| (0x0<<16)              // P4		0 : SEG 21	, 1 :					, 2 :					, 3 :  			, 4 : CS23/ICOM14  
	| (0x0<<12)              // P3		0 : SEG 22	, 1 : EC21		, 2 :					, 3 :  			, 4 : CS22/ICOM15  
	| (0x0<<8)              // P2		0 : SEG 23	, 1 : EC20		, 2 :					, 3 :  			, 4 : CS21/ICOM16  
	| (0x0<<4)              // P1		0 : SEG 24	, 1 : T21O		, 2 : T21C		, 3 :  			, 4 : CS20/ICOM17  
	| (0x0<<0)              // P0		0 : SEG 25	, 1 : T20O		, 2 : T20C		, 3 :  			, 4 : CS19/ICOM18  
	;
	PC->AFSR2 = 0
	| (0x0<<16)              // P12	0 : SEG 13	, 1 : EC11		, 2 :				, 3 : 				, 4 :		
	| (0x0<<12)              // P11	0 : SEG 14	, 1 : EC10		, 2 :				, 3 : 				, 4 :		
	| (0x0<<8)              // P10	0 : SEG 15	, 1 :					, 2 :				, 3 : 				, 4 :		
	| (0x0<<4)              // P9		0 : SEG 16	, 1 :					, 2 :				, 3 : 				, 4 :		
	| (0x0<<0)              // P8		0 : SEG 17	, 1 :					, 2 				, 3 : 				, 4 :		
	;
	
	PC->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PC->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x0<<12)              // P12
	| (0x0<<11)              // P11
	| (0x0<<10)              // P10
	| (0x0<<9)              // P9
	| (0x0<<8)              // P8	
	| (0x0<<7)              // P7
	| (0x0<<6)              // P6
	| (0x0<<5)              // P5
	| (0x0<<4)              // P4
	| (0x0<<3)              // P3
	| (0x0<<2)              // P2
	| (0x0<<1)              // P1
	| (0x0<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - D
	PD->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<10)              // P5
	| (0x01<<8)              // P4
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PD->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PD->AFSR1 = 0
	| (0x00<<20)              // P5		0 : SEG 7	,	1 : 			, 2 : SS11			, 3 : 		, 4 : ICOM8				
	| (0x00<<16)              // P4		0 : SEG 8	,	1 :				, 2 : SCK11			, 3 : 		, 4 : ICOM9
	| (0x00<<12)              // P3		0 : SEG 9	,	1 : RXD11	, 2 : MISO11		, 3 : 		, 4 : ICOM10
	| (0x00<<8)              // P2			0 : SEG 10,	1 : TXD11	, 2 : MOSI11		, 3 : 		, 4 : ICOM11			
	| (0x00<<4)              // P1			0 : SEG 11,	1 : SDA0	, 2 :						, 3 : 		, 4 : ICOM12			 
	| (0x00<<0)              // P0			0 : SEG 12,	1 : SCL0	, 2 :						, 3 : 		, 4 : ICOM13				 
	;
	
	PD->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PD->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - E
	PE->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<30)              // P15
	| (0x01<<28)              // P14
	| (0x01<<26)              // P13
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x01<<8)              // P4
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PE->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<15)              // P15
	| (0x00<<14)              // P14
	| (0x00<<13)              // P13
	| (0x00<<12)              // P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PE->AFSR1 = 0
	| (0x00<<28)              // P7		0 : COM7/SEG4		, 1 : T11O			, 2 : T11C	, 3 :		, 4 : ICOM7	
	| (0x00<<24)              // P6		0 : COM6/SEG3		, 1 : T10O			, 2 : T10C	, 3 :		, 4 : ICOM6	
	| (0x00<<20)              // P5		0 : COM5/SEG2		, 1 : PWM30CB		, 2 :				, 3 :		, 4 : ICOM5	
	| (0x00<<16)              // P4		0 : COM4/SEG1		, 1 : PWM30CA		, 2 :				, 3 :		, 4 : ICOM4	
	| (0x00<<12)              // P3		0 : COM3/SEG0		, 1 : PWM30BB		, 2 :				, 3 :		, 4 : ICOM3	
	| (0x00<<8)              // P2			0 : COM2				, 1 : PWM30BA		, 2 :				, 3 :		, 4 : ICOM2	
	| (0x00<<4)              // P1			0 : COM1				, 1 : PWM30AB		, 2 :				, 3 :		, 4 : ICOM1	
	| (0x00<<0)              // P0			0 : COM0				, 1 : PWM30AA		, 2 :				, 3 :		, 4 : ICOM0	
	;
	PE->AFSR2 = 0
	| (0x00<<28)              // P15		0 : SEG6	, 1 :					, 2 : SS12			, 3 :					, 4 :		
	| (0x00<<24)              // P14		0 : SEG5	, 1 :					, 2 : SCK12			, 3 :					, 4 :		
	| (0x00<<20)              // P13		0 :				, 1 : RXD12		, 2 : MISO12		, 3 :					, 4 :		
	| (0x00<<16)              // P12		0 :				, 1 : TXD12		, 2 : MOSI12		, 3 :					, 4 :		
	| (0x00<<12)              // P11		0 :				, 1 :					, 2 : SS13			, 3 : VLC3		, 4 :		
	| (0x00<<8)              // P10			0 :				, 1 :					, 2 : SCK13			, 3 : VLC2		, 4 :		
	| (0x00<<4)              // P9				0 :				, 1 : RXD13		, 2 : MISO13		, 3 : VLC1		, 4 :		
	| (0x00<<0)              // P8				0 :				, 1 : TXD13		, 2 : MOSI13		, 3 : VLC0		, 4 :		
	;
	
	PE->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<30)              // P15
	| (0x0<<28)              // P14
	| (0x0<<26)              // P13
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PE->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x0<<15)              // P15
	| (0x0<<14)              // P14
	| (0x0<<13)              // P13
	| (0x0<<12)              // P12
	| (0x0<<11)              // P11
	| (0x0<<10)              // P10
	| (0x0<<9)              // P9
	| (0x0<<8)              // P8
	| (0x0<<7)              // P7
	| (0x0<<6)              // P6
	| (0x0<<5)              // P5
	| (0x0<<4)              // P4
	| (0x0<<3)              // P3
	| (0x0<<2)              // P2
	| (0x0<<1)              // P1
	| (0x0<<0)              // P0
	;
	
//////////////////////////////////////////////////////////////////////////////////////////////////////	
	// PORT - F
	PF->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x02<<8)              // P4     clko
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x02<<2)              // P1
	| (0x02<<0)              // P0
	;
	
	PF->TYP = 0					// 0 : Push-pull Output,	1 : Open-drain Output
	| (0x0<<11)              // P11
	| (0x0<<10)              // P10
	| (0x0<<9)              // P9
	| (0x0<<8)              // P8
	| (0x0<<7)              // P7
	| (0x0<<6)              // P6
	| (0x0<<5)              // P5
	| (0x0<<4)              // P4
	| (0x0<<3)              // P3
	| (0x0<<2)              // P2
	| (0x0<<1)              // P1
	| (0x0<<0)              // P0
	;
	
	PF->AFSR1 = 0
	| (0x0<<28)              // P7		0 :				, 1 : T30C		, 2 : SDA0		, 3 :					, 4 :				 
	| (0x0<<24)              // P6		0 :				, 1 : EC30		, 2 : SCL0		, 3 :					, 4 :				 
	| (0x0<<20)              // P5		0 :				, 1 : BLNK		, 2 : INT			, 3 :					, 4 :				 
	| (0x1<<16)              // P4		0 :				, 1 : CLKO		, 2 :					, 3 : R-SET		, 4 :				 
	| (0x0<<12)              // P3		0 :				, 1 : RXD1		, 2 :					, 3 : SXOUT		, 4 :				 
	| (0x0<<8)              // P2		0 :				, 1 : TXD1		, 2 :					, 3 : SXIN		, 4 :				 
	| (0x3<<4)              // P1		0 :				, 1 : SDA1		, 2 :					, 3 : XIN			, 4 :				 
	| (0x3<<0)              // P0		0 :				, 1 : SCL1		, 2 :					, 3 : XOUT		, 4 : ISEG10	 
	;
	PF->AFSR2 = 0
	| (0x0<<12)              // P11	0 : 			, 1 : T14O		, 2 : T14C		, 3 : 				, 4 :				 
	| (0x0<<8)              // P10	0 : 			, 1 : T13O		, 2 : T13C		, 3 : 				, 4 :				 
	| (0x0<<4)              // P9		0 : 			, 1 : EC14		, 2 :					, 3 : 				, 4 :				 
	| (0x0<<0)              // P8		0 : 			, 1 : EC13		, 2 :					, 3 : 				, 4 :				
	;
	
	PF->PUPD = 0              // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PF->OUTDR = 0              // 0 : Output Low,	1 : Output High
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8	
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
//	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}
